# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import logging
import os
import re
import socket
import types
from sys import modules
from typing import cast

from vdk.api.plugin.hook_markers import hookimpl
from vdk.internal.builtin_plugins.config import vdk_config
from vdk.internal.builtin_plugins.config.vdk_config import LOG_LEVEL_VDK
from vdk.internal.builtin_plugins.run.job_context import JobContext
from vdk.internal.core import errors
from vdk.internal.core.config import ConfigurationBuilder
from vdk.internal.core.errors import ResolvableBy
from vdk.internal.core.statestore import CommonStoreKeys

SYSLOG_URL = "SYSLOG_URL"
SYSLOG_PORT = "SYSLOG_PORT"
SYSLOG_SOCK_TYPE = "SYSLOG_SOCK_TYPE"
SYSLOG_ENABLED = "SYSLOG_ENABLED"

SYSLOG_SOCK_TYPE_VALUES_DICT = {"UDP": socket.SOCK_DGRAM, "TCP": socket.SOCK_STREAM}


def _parse_log_level_module(log_level_module):
    valid_logging_levels = [
        "NOTSET",
        "DEBUG",
        "INFO",
        "WARN",
        "WARNING",
        "ERROR",
        "FATAL",
        "CRITICAL",
    ]
    try:
        if log_level_module and log_level_module.strip():
            modules = log_level_module.split(";")
            result = {}
            for module in modules:
                if module:
                    module_and_level = module.split("=")
                    if not re.search("[a-zA-Z0-9_.-]+", module_and_level[0].lower()):
                        raise ValueError(
                            f"Invalid logging module name: '{module_and_level[0]}'. "
                            f"Must be alphanumerical/underscore characters."
                        )
                    if module_and_level[1].upper() not in valid_logging_levels:
                        raise ValueError(
                            f"Invalid logging level: '{module_and_level[1]}'. Must be one of {valid_logging_levels}."
                        )
                    result[module_and_level[0]] = {"level": module_and_level[1].upper()}
            return result
        else:
            return {}
    except Exception as e:
        errors.report_and_throw(
            errors.VdkConfigurationError(
                "Invalid logging configuration passed to LOG_LEVEL_MODULE.",
                f"Error is: {e}. log_level_module was set to {log_level_module}.",
                "Set correctly configuration to log_level_debug configuration in format 'module=level;module2=level2'",
            )
        )


def configure_initial_logging_before_anything():
    """
    Configure logging at the start of the app.
    At this point we do not know what user has configured so we use some sensible approach
    All logs would be vdk only logs. User code will not have been executed yet so we will use log_level_vdk
    to control the level.
    We default to WARN since in most cases users would not care about internal vdk logs.

    Logging lifecycle:
    1. This function is executed at the entry point of the app and configures default logging format and level
    2. After vdk_initialize hook is executed and CLI command starts then `vdk --verbosity` option is taken (if set)
        and click logging configuration takes over
    3. When initialize_job hook (applicable for vdk run only) is executed then below configure_loggers function
        is run which adds more context to the logs and initializes syslog handler (if configured to do so)
    """
    log_level = "WARNING"
    if os.environ.get(LOG_LEVEL_VDK, None):
        log_level = os.environ.get(LOG_LEVEL_VDK)
    elif os.environ.get("VDK_LOG_LEVEL_VDK", None):
        log_level = os.environ.get("VDK_LOG_LEVEL_VDK")

    logging.basicConfig(format="%(message)s", level=logging.getLevelName(log_level))


def configure_loggers(
    job_name: str = "",
    attempt_id: str = "no-id",
    log_config_type: str = None,
    vdk_logging_level: str = "DEBUG",
    log_level_module: str = None,
    syslog_args: (str, int, str, bool) = ("localhost", 514, "UDP", False),
) -> None:
    """
    Configure default logging configuration

    :param job_name: the name of the job
    :param attempt_id: the id of the current job run attempt
    :param log_config_type: where the job is executed: CLOUD or LOCAL
    :param vdk_logging_level: The level for vdk specific logs.
    :param log_level_module: The level for modules specific logs
    :param syslog_args: Arguments necessary for SysLog logging.
    """

    import logging.config

    if _is_already_configured():
        log = logging.getLogger(__name__)
        msg = "Logging already configured. This seems like a bug. Fix me!."
        # PLugins would override loggin directly wiht logging library not through this method
        # it is probably OK for tests to configure logging twice. VDK, however, shouldn't.
        log.warning(msg)

    # Likely most logging system (like Log Insight) show the hostname from where the syslog message arrived so no need to include it here.
    DETAILED_FORMAT = (
        f"%(asctime)s [VDK] {job_name} [%(levelname)-5.5s] %(name)-30.30s %(filename)20.20s:%("
        f"lineno)-4.4s %(funcName)-16.16s[id:{attempt_id}]- %(message)s"
    )

    _LOGGERS = {
        "requests_kerberos": {"level": "INFO"},
        "requests_oauthlib": {"level": "INFO"},
        "urllib3": {"level": "INFO"},
        "vdk": {"level": vdk_logging_level},
    }
    _LOGGERS.update(_parse_log_level_module(log_level_module))

    _FORMATTERS = {"detailedFormatter": {"format": DETAILED_FORMAT}}

    _CONSOLE_HANDLER = {
        "class": "logging.StreamHandler",
        "formatter": "detailedFormatter",
        "stream": "ext://sys.stderr",
    }

    syslog_url, syslog_port, syslog_sock_type, syslog_enabled = syslog_args

    if syslog_sock_type not in SYSLOG_SOCK_TYPE_VALUES_DICT:
        errors.report_and_throw(
            errors.VdkConfigurationError(
                f"Provided configuration variable for {SYSLOG_SOCK_TYPE} has invalid value.",
                f"VDK was run with {SYSLOG_SOCK_TYPE}={syslog_sock_type}, however {syslog_sock_type} is invalid value for this variable.",
                f"Provide a valid value for {SYSLOG_SOCK_TYPE}."
                f"Currently possible values are {list(SYSLOG_SOCK_TYPE_VALUES_DICT.keys())}",
            )
        )

    _SYSLOG_HANDLER = {
        "level": "DEBUG",
        "class": "logging.handlers.SysLogHandler",
        "formatter": "detailedFormatter",
        "address": (syslog_url, syslog_port),
        "socktype": SYSLOG_SOCK_TYPE_VALUES_DICT[syslog_sock_type],
        "facility": "user",
    }

    handlers = ["consoleHandler"]
    if syslog_enabled:
        handlers.append("SysLog")

    if "CLOUD" == log_config_type:
        CLOUD = {  # @UnusedVariable
            "version": 1,
            "handlers": {"consoleHandler": _CONSOLE_HANDLER, "SysLog": _SYSLOG_HANDLER},
            "formatters": _FORMATTERS,
            "root": {"handlers": handlers},
            "loggers": _LOGGERS,
            "disable_existing_loggers": False,
        }
        logging.config.dictConfig(CLOUD)
    elif "NONE" == log_config_type:
        pass
    else:
        LOCAL = {  # @UnusedVariable
            "version": 1,
            "handlers": {"consoleHandler": _CONSOLE_HANDLER, "SysLog": _SYSLOG_HANDLER},
            "formatters": _FORMATTERS,
            "root": {"handlers": handlers},
            "loggers": _LOGGERS,
            "disable_existing_loggers": False,
        }
        logging.config.dictConfig(LOCAL)
    _set_already_configured()


def _set_already_configured() -> None:
    setattr(modules[__name__], "logging_already_configured", True)


def _is_already_configured() -> bool:
    res = hasattr(modules[__name__], "logging_already_configured")
    return res


def ensure_logging_for_current_test() -> None:
    """
    To be called from test code only.

    Ensures that logging is configured, either by another class, or by the caller.
    Useful when debugging a single test class, not the whole suite (assuming all suites configure logging)

    Uses filename of caller for derived_dataset parameter, e.g. if this method is called from impala_connection_test.py
    then logs would be configured to look like:

    2018-04-19 16:05:48,488=1524143148[VDK] impala_connection_test.py [DEBUG]  ......
    """
    if not _is_already_configured():
        try:
            from inspect import currentframe

            me = cast(types.FrameType, currentframe())
            caller = cast(types.FrameType, me.f_back)
            path = caller.f_locals["__file__"]
            from os.path import split

            dir, fname = split(path)  # @ReservedAssignment
        except:
            fname = "n/a"
        from time import time

        configure_loggers(fname, str(time()))


class LoggingPlugin:
    """
    Define the logging plugin
    """

    @staticmethod
    @hookimpl
    def vdk_configure(config_builder: ConfigurationBuilder):
        config_builder.add(
            key=SYSLOG_URL,
            default_value="localhost",
            description="The hostname of the endpoint to which VDK logs will be sent through SysLog.",
        )
        config_builder.add(
            key=SYSLOG_PORT,
            default_value=514,
            description="The port of the endpoint to which VDK logs will be sent through SysLog.",
        )
        config_builder.add(
            key=SYSLOG_ENABLED,
            default_value=False,
            description="If set to True, SysLog log forwarding is enabled.",
        )
        config_builder.add(
            key=SYSLOG_SOCK_TYPE,
            default_value="UDP",
            description=f"Socket type for SysLog log forwarding connection. Currently possible values are {list(SYSLOG_SOCK_TYPE_VALUES_DICT.keys())}",
        )

    @hookimpl
    def initialize_job(self, context: JobContext) -> None:
        """
        Initialize logging for running Data Job.

        When running a data job logging is more verbose, it includes more details like op id, job name.
        """

        attempt_id = context.core_context.state.get(CommonStoreKeys.ATTEMPT_ID)
        job_name = context.name
        log_config_type = context.core_context.configuration.get_value(
            vdk_config.LOG_CONFIG
        )
        vdk_log_level = context.core_context.configuration.get_value(
            vdk_config.LOG_LEVEL_VDK
        )
        log_level_module = context.core_context.configuration.get_value(
            vdk_config.LOG_LEVEL_MODULE
        )
        syslog_url = context.core_context.configuration.get_value(SYSLOG_URL)
        syslog_port = context.core_context.configuration.get_value(SYSLOG_PORT)
        syslog_sock_type = context.core_context.configuration.get_value(
            SYSLOG_SOCK_TYPE
        )
        syslog_enabled = context.core_context.configuration.get_value(SYSLOG_ENABLED)
        try:  # If logging initialization fails we want to attempt sending telemetry before exiting VDK
            configure_loggers(
                job_name,
                attempt_id,
                log_config_type=log_config_type,
                vdk_logging_level=vdk_log_level,
                log_level_module=log_level_module,
                syslog_args=(syslog_url, syslog_port, syslog_sock_type, syslog_enabled),
            )
            log = logging.getLogger(__name__)
            log.debug(f"Initialized logging for log type {log_config_type}.")
        except Exception as e:
            # Have local logs on DEBUG level, when standard log configuration fails
            logging.basicConfig(level=logging.DEBUG)
            log = logging.getLogger(__name__)
            log.warning(
                "Logging configuration %s failed.\n"
                "Trying to send telemetry for Job attempt: %s"
                % (log_config_type, attempt_id)
            )
            errors.log_exception(
                logging.getLogger(__name__),
                e,
                "Failed to initialize logging",
                errors.MSG_WHY_FROM_EXCEPTION(e),
                "Failed to initialize data job logging."
                " Will proceed with basic local logging on"
                " DEBUG level.",
                "Depending on stacktrace.",
            )



# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import logging
from typing import Callable
from typing import Dict
from typing import Union

from vdk.api.plugin.plugin_input import IManagedConnectionRegistry
from vdk.internal.builtin_plugins.config.vdk_config import DB_DEFAULT_TYPE
from vdk.internal.builtin_plugins.connection.connection_hooks import (
    ConnectionHookSpecFactory,
)
from vdk.internal.builtin_plugins.connection.impl.wrapped_connection import (
    WrappedConnection,
)
from vdk.internal.builtin_plugins.connection.managed_connection_base import (
    ManagedConnectionBase,
)
from vdk.internal.builtin_plugins.connection.pep249.interfaces import PEP249Connection
from vdk.internal.core import errors
from vdk.internal.core.config import Configuration

log = logging.getLogger(__name__)


class ManagedConnectionRouter(IManagedConnectionRegistry):
    """
    Create ManagedConnection by routing the configured database plugin.
    Configuration is controlled by DB_DEFAULT_TYPE for default connection.
    Or specific connection can be specified by open_connection(dbtype)
    In both cases dbtype must match the string in which the plugin register itself with.
    """

    def __init__(
        self,
        cfg: Configuration,
        connection_hook_spec_factory: ConnectionHookSpecFactory,
    ):
        self._cfg: Configuration = cfg
        self._connection_hook_spec_factory = connection_hook_spec_factory
        self._log: logging.Logger = logging.getLogger(__name__)
        self._connections: Dict[str, ManagedConnectionBase] = dict()
        self._connection_builders: Dict[
            str, Callable[[], ManagedConnectionBase]
        ] = dict()

    def add_open_connection_factory_method(
        self,
        dbtype: str,
        open_connection_func: Callable[
            [], Union[ManagedConnectionBase, PEP249Connection]
        ],
    ) -> None:
        """
        Add new connection factory method. See parent doc for more.
        """
        self._connection_builders[dbtype.lower()] = open_connection_func

    def open_default_connection(self) -> ManagedConnectionBase:
        """
        Open connection to the database configured as default (by db_default_type option).
        """
        dbtype = self._cfg.get_value(DB_DEFAULT_TYPE)
        if dbtype is None:
            if len(self._connection_builders) == 1:
                dbtype = list(self._connection_builders.keys())[0]
                self._log.debug(
                    f"DB_DEFAULT_TYPE has not been set. Found one database supported: {dbtype}. Will use it as default"
                )

        return self.open_connection(dbtype)

    def open_connection(self, dbtype: str) -> ManagedConnectionBase:
        """
        Opens a connection for the given database type.

        :param dbtype: The type of connection to open. It needs to have been registered before that by add_connection_builder
        or it will thrown an error
        :return: the new connection if successful or throws an exception
        """
        dbtype = dbtype.lower() if dbtype else None
        conn = None
        if dbtype in self._connections:
            conn = self._connections[dbtype]
        elif dbtype in self._connection_builders:
            self._log.debug(f"Connection to {dbtype} is missing. Will try to connect")
            conn = self.__create_connection(dbtype)
        else:
            errors.report_and_throw(
                errors.VdkConfigurationError(
                    f"Provided configuration variable for {DB_DEFAULT_TYPE} has invalid value.",
                    f"VDK was run with {DB_DEFAULT_TYPE}={dbtype}, however {dbtype} is invalid value for this variable.",
                    f"Provide either valid value for {DB_DEFAULT_TYPE} or install database plugin that supports this type. "
                    f"Currently possible values are {list(self._connection_builders.keys())}",
                )
            )
        if not conn._is_connected():
            conn.connect()
        return conn

    def __cache_connection(self, dbType: str, conn: ManagedConnectionBase):
        if dbType in self._connections:
            # Though not expected, this checks is added to reduce the chance of connection leaks.
            log.warning(
                f"There is already cached connection for dbType {dbType}. Replacing it "
            )
            prev_conn = self._connections[dbType]
            try:
                prev_conn.close()
            except Exception as e:
                log.debug(
                    f"Failed to close cached connection: {e}. Likely it's no longer valid."
                )
        self._connections[dbType] = conn

    def __create_connection(self, dbtype: str):
        conn = self._connection_builders[dbtype]()
        if isinstance(conn, ManagedConnectionBase):
            self.__cache_connection(dbtype, conn)
            if not conn._connection_hook_spec_factory:
                conn._connection_hook_spec_factory = self._connection_hook_spec_factory
        elif conn is None:
            errors.report_and_throw(
                errors.VdkConfigurationError(
                    f"Could not create new connection of db type {dbtype}.",
                    f"VDK was run with {DB_DEFAULT_TYPE}={dbtype}, however no valid connection was created.",
                    f"Seems to be a bug in the plugin for dbtype {dbtype}. Make sure it's correctly installed. "
                    f"If upgraded recently consider reverting to previous version. Or use another db type. "
                    f"Currently possible values are {list(self._connection_builders.keys())}",
                )
            )
        else:
            log = logging.getLogger(conn.__class__.__name__)
            # we will let ManagedConnection to open it when needed.
            conn.close()
            wrapped_conn = WrappedConnection(
                log,
                self._connection_builders[dbtype],
                self._connection_hook_spec_factory,
            )
            self.__cache_connection(dbtype, wrapped_conn)

        return self._connections[dbtype]



# Copyright 2021-2023 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
"""
errors -- Exception handling implementations

errors is a module that handles all errors.
It defines classes and methods for handling exceptions, and ensuring that there are two types of exceptions:
 - infrastructure-related: should be handled by Platform team.
 - user code-related: should be handled by the owner of the DataJob.
"""
from __future__ import annotations

from logging import Logger

from vdk.internal.core.error_classifiers import *

# Due to the expansion of this file's responsibilities over time,
# we've decided to reorganize its contents for better maintainability.
# The logic previously contained here has been moved to a separate module. However, to ensure backward compatibility,
# we are importing all the elements from the new module here.
# This allows existing code to continue functioning without disruption.

log = logging.getLogger(__name__)


# ERROR TYPES


def plain_vdk_error_formatter(ex: BaseVdkError):
    """
    Joins every line of the exception message using space as delimiter.
    This includes the header.

    Example (lines wrapped for convenience):

        vdk.internal.core.errors.VdkConfigurationError: VdkConfigurationError: An error of resolvable type ResolvableByActual.PLATFORM occurred
        Provided configuration variable for DB_DEFAULT_TYPE has invalid value. VDK was run with DB_DEFAULT_TYPE=sqlite, however sqlite is invalid value for this variable.
        I'm rethrowing this exception to my caller to process.Most likely this will result in failure of current Data Job. Provide either valid value for DB_DEFAULT_TYPE or
        install database plugin that supports this type. Currently possible values are []
    """
    ex._lines[0] = ex._lines[0] + "\n"
    ex._pretty_message = " ".join(ex._lines)


def pretty_vdk_error_formatter(ex: BaseVdkError):
    """
    Creates a box with a header and body around the error message.
    The header contains the error class name, the error type and
    who should resolve the error. The body contains the actual error
    message. The box length is determined by the header lenght. Lines
    in the body that are longer than the header are wrapped on the
    first space before the max length cutoff.

    Example:

        +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        +   VdkConfigurationError: An error of type Configuration error occurred. Error should be fixed by Platform   +
        +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        +  Provided configuration variable for DB_DEFAULT_TYPE has invalid value.                                     +
        +  VDK was run with DB_DEFAULT_TYPE=sqlite, however sqlite is invalid value for this variable.                +
        +  I'm rethrowing this exception to my caller to process.Most likely this will result in failure of           +
        +  current Data Job.                                                                                          +
        +  Provide either valid value for DB_DEFAULT_TYPE or install database plugin that supports this type.         +
        +  Currently possible values are []                                                                           +
        +                                                                                                             +
        +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    """
    box_char = "+"
    lines = ex._lines
    header = lines[0]
    max_len = len(header)
    # Wrap the message lines
    wrapped_lines = []
    for i in range(1, len(lines)):
        line = lines[i]
        # Find the nearest space if a line is
        # longer than the header and wrap around it
        # Do this until the remainder of the line is shorter
        # than the header
        if len(line) > max_len:
            l = line
            while len(l) > max_len:
                br = max_len
                while l[br] != " " and br > 0:
                    br -= 1
                if br == 0:
                    br = max_len
                wrapped_lines.append(l[: br + 1])
                l = l[br + 1 :]
            wrapped_lines.append(l)
        else:
            wrapped_lines.append(line)
    # build th header
    header = box_char + header.center(max_len + 6) + box_char
    # build the lines with the box char
    lines = [box_char + "  " + s.ljust(max_len + 4) + box_char for s in wrapped_lines]
    # build the box sides
    side = (max_len + 8) * box_char
    ex._pretty_message = "\n".join(["\n" + side, header, side, "\n".join(lines), side])


class BaseVdkError(Exception):
    def __init__(
        self,
        vdk_resolvable_actual: ResolvableByActual = None,
        resolvable_by: ResolvableBy = None,
        *error_message_lines: str,
    ):
        """
        :param vdk_resolvable_actual: who should resolve the error (User or Platform Team)
        :param resolvable_by: the vdk error type, e.g. Platform, User or Config error
        :param *error_message_lines: optional, the error message lines used to build the error representation
        """

        # Check if error message or dict was passed
        # for compatibility with vdk plugins
        self._lines = []
        header = f"{self.__class__.__name__}: An error of resolvable type {vdk_resolvable_actual} occurred"
        self._lines.append(header)
        if error_message_lines and isinstance(error_message_lines[0], ErrorMessage):
            message = error_message_lines[0]
            self._lines.extend(
                [
                    message.summary,
                    message.what,
                    message.why,
                    message.consequences,
                    message.countermeasures,
                ]
            )
        elif error_message_lines and isinstance(error_message_lines[0], dict):
            message = error_message_lines[0]
            self._lines.extend(message.values())
        else:
            self._lines.extend(error_message_lines)
        super().__init__(str(self._lines))

        self._vdk_resolvable_by = resolvable_by
        self._vdk_resolvable_actual = vdk_resolvable_actual
        self._pretty_message = " ".join(self._lines)

    def __str__(self):
        return self._pretty_message

    def __repr__(self):
        return self._pretty_message

    def _format(self, formatter):
        formatter(self)


class SkipRemainingStepsException(BaseVdkError):
    """
    An exception used to skip the remaining steps of a Data Job.

    When this exception is thrown from a data job python step, all the other steps will
    be skipped and the data job execution will exit and be marked as success.
    """

    def __init__(self, *error_message_lines: str):
        super().__init__(None, None, *error_message_lines)


# REPORTING


def log_exception(log: Logger, exception: BaseException, *lines: str) -> None:
    """
    Log message and exception
    """
    message = "\n".join([s for s in lines if s])
    log.warning(message)
    log.exception(exception)


def report(error_type: ResolvableBy, exception: BaseException):
    set_exception_resolvable_by(exception, error_type)
    resolvable_context().add(
        Resolvable(
            error_type,
            get_exception_resolvable_by_actual(exception),
            str(exception),
            exception,
        )
    )


def report_and_throw(
    exception: BaseVdkError, resolvable_by: ResolvableBy = None
) -> None:
    """
    Add exception to resolvable context and then throw it to be handled up the stack.
    """
    if resolvable_by:
        set_exception_resolvable_by(exception, resolvable_by)

    resolvable_context().add(
        Resolvable(
            get_exception_resolvable_by(exception),
            get_exception_resolvable_by_actual(exception),
            str(exception),
            exception,
        )
    )
    raise exception


def report_and_rethrow(error_type: ResolvableBy, exception: BaseException) -> None:
    """
    Add exception to resolvable context and then throw it to be handled up the stack.
    """
    report(error_type, exception)
    raise exception


# COMPATIBILITY


def log_and_throw(
    to_be_fixed_by: ResolvableBy,
    log: Logger,
    what_happened: str,
    why_it_happened: str,
    consequences: str,
    countermeasures: str,
) -> None:
    """
    Deprecated: "Use report_and_throw and do the logging separately"

    log_and_throw kept for compatibility of plugins with older version of vdk-core
    """
    message = [what_happened, why_it_happened, consequences, countermeasures]
    if to_be_fixed_by == ResolvableBy.USER_ERROR:
        report_and_throw(UserCodeError(*message))
    if to_be_fixed_by == ResolvableBy.PLATFORM_ERROR:
        report_and_throw(PlatformServiceError(*message))
    if to_be_fixed_by == ResolvableBy.CONFIG_ERROR:
        report_and_throw(VdkConfigurationError(*message))


def log_and_rethrow(
    to_be_fixed_by: ResolvableBy,
    log: Logger,
    what_happened: str,
    why_it_happened: str,
    consequences: str,
    countermeasures: str,
    exception: BaseException,
    wrap_in_vdk_error=False,
) -> None:
    """
    Deprecated: Use report_and_rethrow and do the logging separately

    log_and_rethrow kept for compatibility of plugins with older version of vdk-core
    """
    if wrap_in_vdk_error:
        pass
        # wrap
    message = [what_happened, why_it_happened, consequences, countermeasures]
    log.error("\n".join(message))
    report_and_rethrow(to_be_fixed_by, exception)


class ErrorMessage:
    """
    Deprecated: Pass the error message lines to the vdk exception directly

    Standard format for Error messages in VDK. Use it when throwing exceptions or logging error level.
    """

    def __init__(
        self, summary: str, what: str, why: str, consequences: str, countermeasures: str
    ):
        self.summary = summary
        self.what = what
        self.why = why
        self.consequences = consequences
        self.countermeasures = countermeasures

    def _get_template(self, replace_with: str) -> str:
        return "{}r%15s : {}r%15s : {}r%15s : {}r%15s : {}".replace(
            "r", replace_with
        ) % (
            "what happened".upper(),
            "why it happened".upper(),
            "consequences".upper(),
            "countermeasures".upper(),
        )

    def _to_string(self, template: str) -> str:
        return template.format(
            self.summary, self.what, self.why, self.consequences, self.countermeasures
        )

    def __str__(self) -> str:
        return self._to_string(self._get_template("\n"))

    def to_html(self) -> str:
        """
        :return: html representaiton of the error.
        """
        return self._to_string(self._get_template("<br />"))


class DomainError(BaseVdkError):
    """
    Deprecated: Use BaseVdkError directly or extend BaseVdkError into a domain-specific class
    """

    pass


class PlatformServiceError(BaseVdkError):
    """
    Deprecated: Use BaseVdkError directly or extend BaseVdkError into a domain-specific class

    Error caused by issue in Platform

    Service error are those errors which indicate for bad condition of the service. which prevent the normal
    completion of given request. User can do little more beyond retrying.
    """

    def __init__(self, *error_message_lines: str):
        super().__init__(
            ResolvableByActual.PLATFORM,
            ResolvableBy.PLATFORM_ERROR,
            *error_message_lines,
        )


class VdkConfigurationError(BaseVdkError):
    """
    Deprecated: Use BaseVdkError directly or extend BaseVdkError into a domain-specific class

    An exception in the configuration of the Data Job

    If job is executed on user (local execution) premises - this is considered #RESOLVED_BY_JOB_OWNER
    Otherwise If job is executed in "cloud" - this is considered #RESOLVED_BY_PLATFORM class of error
    """

    def __init__(self, *error_message_lines: str):
        super().__init__(
            CONFIGURATION_ERRORS_ARE_TO_BE_RESOLVED_BY,
            ResolvableBy.CONFIG_ERROR,
            *error_message_lines,
        )


class UserCodeError(BaseVdkError):
    """
    Deprecated: Use BaseVdkError directly or extend BaseVdkError into a domain-specific class

    Error caused by issue in user code
    """

    def __init__(self, *error_message_lines: str):
        super().__init__(
            ResolvableByActual.USER, ResolvableBy.USER_ERROR, *error_message_lines
        )




RELEASE_VERSION='0.3.1087709269'
BUILD_DATE='Tue Nov 28 12:34:06 UTC 2023'
BUILD_MACHINE_INFO='Linux runner-bhjdzkxh-project-28359933-concurrent-0hkpbr 5.4.235-144.344.amzn2.x86_64 #1 SMP Sun Mar 12 12:50:22 UTC 2023 x86_64 GNU/Linux'
GITLAB_CI_JOB_ID='5632774470'
GIT_COMMIT_SHA='709070bb01ac7755501ef7774537fd0e217af3bd'
GIT_BRANCH='main'
